#version 330

in vec2 texCoord;

uniform sampler2D texUnit;
uniform sampler2D kernelUnit;
uniform ivec2 kernelSize = ivec2(5, 5);
uniform bool grayscale = false;
uniform bool normalize = true;

out vec4 outputF;

//----------------------------------------------------------------
// TODO:
// 1) Add a second sampler, kernel size, grayscale and normalize uniforms.
// 2) Return to using the texture function to allow wrapped out-of-range accesses.
// 3) Multiply the kernel with the neighbourhood of each pixel.
//		- Load either the texel or convert its value to grayscale.
//		- Load the weights from the red channel.
//		- Normalize the sum by the sum of weights only if required.
// 4) Compute the convolved image.
//----------------------------------------------------------------

//----------------------------------------------------------------
// Solved problems:
// 1) Blackening around the edges.
// 2) Different codes for different sizes.
// 3) Need for various coeficients.
//----------------------------------------------------------------

void main()
{
	ivec2 texSize = textureSize(texUnit, 0);
	ivec2 ofs;
	ivec2 kernelSizeHalf = kernelSize/2;
	vec2 scale = 1.f / vec2(texSize);
	vec4 sum = vec4(0.f);
	float sumWeight = 0.f;

	for(ofs.y = -kernelSizeHalf.y; ofs.y <= kernelSizeHalf.y; ofs.y++)
	{
		for(ofs.x = -kernelSizeHalf.x; ofs.x <= kernelSizeHalf.x; ofs.x++)
		{
			vec4 texel;
			if(!grayscale)
			{
				texel = texture(texUnit, texCoord+ofs*scale, 0);
			}
			else
			{
				vec3 NTSCWeights = vec3(0.299f, 0.587f, 0.114f);
				vec4 tex = texture(texUnit, texCoord+ofs*scale, 0);
				float gray = dot(tex.rgb, NTSCWeights);
				texel = vec4(gray, gray, gray, tex.a);
			}

			float weight = texelFetch(kernelUnit, ofs+kernelSizeHalf, 0).r; // Beware coordinates order
			sum += texel * weight;
			sumWeight += weight;
		}
	}
	
	if(normalize)
		outputF = sum/sumWeight;
	else
		outputF = sum;
}